# Copyright 2025 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""Check the content of a bundle according to a manifest."""

import sys
import os


def write_if_needed(path, content):
  """Write `content` to `path` if needed."""
  if os.path.isfile(path):
    # If the file exists and has the same content, skip writing.
    with open(path, encoding='utf-8') as file:
      if content == file.read():
        return
  with open(path, 'w', encoding='utf-8') as file:
    file.write(content)
    file.flush()



def read_manifest(manifest_path):
  """Read a manifest file."""
  manifest_content = set()
  with open(manifest_path, encoding='utf-8') as manifest_file:
    for line in manifest_file:
      assert line.endswith('\n')
      manifest_content.add(line[:-1])
  return manifest_content


def main(args):
  if len(args) != 3:
    print(
      f'usage: check-manifest.py bundle-dir manifest-path output-path',
      file=sys.stderr)
    return 1

  bundle_dir, manifest_path, output_path = args
  if not os.path.isdir(bundle_dir):
    print(
      f'error: {bundle_dir}: not a directory',
      file=sys.stderr)
    return 1

  has_unexpected_files = False
  manifest_content = read_manifest(manifest_path)
  for dirpath, dirnames, filenames in os.walk(bundle_dir):
    dirnames_to_skip = []
    for dirname in dirnames:
      subdirpath = os.path.relpath(os.path.join(dirpath, dirname), bundle_dir)
      if subdirpath in manifest_content:
        dirnames_to_skip.append(dirname)
    if dirnames_to_skip:
      dirnames[:] = list(set(dirnames) - set(dirnames_to_skip))
    for filename in filenames:
      filepath = os.path.relpath(os.path.join(dirpath, filename), bundle_dir)
      if filepath not in manifest_content:
        fullpath = os.path.normpath(os.path.join(bundle_dir, filepath))
        print(f'error: {fullpath}: unexpected file', file=sys.stderr)
        has_unexpected_files = True

  write_if_needed(output_path, 'ko\n' if has_unexpected_files else 'ok\n')

  if has_unexpected_files:
    return 1

  return 0

if __name__ == '__main__':
  sys.exit(main(sys.argv[1:]))



# Copyright 2018 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

DEPS = [
    'windows_sdk',
    'recipe_engine/platform',
    'recipe_engine/properties',
    'recipe_engine/step',
]


def RunSteps(api):
  with api.windows_sdk():
    api.step('gn', ['gn', 'gen', 'out/Release'])
    api.step('ninja', ['ninja', '-C', 'out/Release'])


def GenTests(api):
  for platform in ('linux', 'mac', 'win'):
    properties = {
        'buildername': 'test_builder',
    }
    yield (api.test(platform) + api.platform.name(platform) +
           api.properties.generic(**properties))



#!/usr/bin/env python3
# Copyright 2018 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

import os
import shutil
import subprocess
import sys
import timeit


IS_WIN = sys.platform.startswith('win')


def RemoveDir(d):
  if os.path.isdir(d):
    shutil.rmtree(d)


def Trial(gn_path_to_use, save_out_dir=None):
  bin_path = os.path.join('out', 'gntrial')
  if not os.path.isdir(bin_path):
    os.makedirs(bin_path)
  gn_to_run = os.path.join(bin_path, 'gn' + ('.exe' if IS_WIN else ''))
  shutil.copy2(gn_path_to_use, gn_to_run)
  comp_dir = os.path.join('out', 'COMP')
  subprocess.check_call([gn_to_run, 'gen', comp_dir, '-q', '--check'])
  if save_out_dir:
    RemoveDir(save_out_dir)
    shutil.move(comp_dir, save_out_dir)


def main():
  if len(sys.argv) < 3 or len(sys.argv) > 4:
    print('Usage: full_test.py /chrome/tree/at/762a25542878 rel_gn_path [clean]')
    return 1

  if len(sys.argv) == 4:
    RemoveDir('out')

  subprocess.check_call([sys.executable, os.path.join('build', 'gen.py')])
  subprocess.check_call(['ninja', '-C', 'out'])
  subprocess.check_call([os.path.join('out', 'gn_unittests')])
  orig_dir = os.getcwd()

  in_chrome_tree_gn = sys.argv[2]
  our_gn = os.path.join(orig_dir, 'out', 'gn' + ('.exe' if IS_WIN else ''))

  os.chdir(sys.argv[1])

  # Check in-tree vs. ours. Uses:
  # - Chromium tree at 762a25542878 in argv[1] (this can be off by a bit, but
  #   is roughly when GN was moved out of the Chrome tree, so matches in case GN
  #   semantics/ordering change after that.)
  # - relative path to argv[1] built gn binary in argv[2]

  # First, do a comparison to make sure the output between the two gn binaries
  # actually matches.
  print('Confirming output matches...')
  dir_a = os.path.join('out', 'a')
  dir_b = os.path.join('out', 'b')
  Trial(in_chrome_tree_gn, dir_a)
  Trial(our_gn, dir_b)
  subprocess.check_call(['diff', '-r', dir_a, dir_b])

  # Then, some time trials.
  TRIALS = 5
  print('Comparing performance... (takes a while)')
  time_a = timeit.timeit('Trial("%s")' % in_chrome_tree_gn, number=TRIALS,
                         setup='from __main__ import Trial')
  time_b = timeit.timeit('Trial("%s")' % our_gn, number=TRIALS,
                         setup='from __main__ import Trial')
  print('In-tree gn avg: %.3fs' % (time_a / TRIALS))
  print('Our gn avg: %.3fs' % (time_b / TRIALS))

  return 0


if __name__ == '__main__':
    sys.exit(main())



# Copyright 2018 The Chromium Authors. All rights reserved.
# Use of this source code is governed under the Apache License, Version 2.0
# that can be found in the LICENSE file.
"""Recipe for building GN."""

from recipe_engine.recipe_api import Property

DEPS = [
    'recipe_engine/buildbucket',
    'recipe_engine/cas',
    'recipe_engine/cipd',
    'recipe_engine/context',
    'recipe_engine/file',
    'recipe_engine/json',
    'recipe_engine/path',
    'recipe_engine/platform',
    'recipe_engine/properties',
    'recipe_engine/raw_io',
    'recipe_engine/step',
    'target',
    'macos_sdk',
    'windows_sdk',
]

PROPERTIES = {
    'repository': Property(kind=str, default='https://gn.googlesource.com/gn'),
}

# On select platforms, link the GN executable against jemalloc for a drastic speed boost.
JEMALLOC_GIT_URL = 'https://fuchsia.googlesource.com/third_party/github.com/jemalloc/jemalloc.git'
JEMALLOC_TAG = '5.3.0'

def _get_libcxx_include_path(api):
  # Run the preprocessor with an empty input and print all include paths.
  lines = api.step(
      'xcrun toolchain', [
          'xcrun', '--toolchain', 'clang', 'clang++', '-xc++', '-fsyntax-only',
          '-Wp,-v', '/dev/null'
      ],
      stderr=api.raw_io.output_text(name='toolchain', add_output_log=True),
      step_test_data=lambda: api.raw_io.test_api.stream_output_text(
          str(api.macos_sdk.sdk_dir.join('include', 'c++', 'v1')),
          stream='stderr')).stderr.splitlines()
  # Iterate over all include paths and look for the SDK libc++ one.
  sdk_dir = str(api.macos_sdk.sdk_dir)
  for line in lines:
    line = line.strip()
    if line.startswith(sdk_dir) and 'include/c++/v1' in line:
      return line
  return None  # pragma: no cover


def _get_compilation_environment(api, target, cipd_dir):
  if target.is_linux:
    triple = '--target=%s' % target.triple
    if target.triple == 'riscv64-linux-gnu':
      sysroot = '--sysroot=%s' % cipd_dir.join('sysroot-focal')
    else:
      sysroot = '--sysroot=%s' % cipd_dir.join('sysroot')
    env = {
        'CC': cipd_dir.join('bin', 'clang'),
        'CXX': cipd_dir.join('bin', 'clang++'),
        'AR': cipd_dir.join('bin', 'llvm-ar'),
        'CFLAGS': '%s %s' % (triple, sysroot),
        'LDFLAGS': '%s %s -static-libstdc++' % (triple, sysroot),
    }
  elif target.is_mac:
    triple = '--target=%s' % target.triple
    sysroot = '--sysroot=%s' % api.step(
        'xcrun sdk-path', ['xcrun', '--show-sdk-path'],
        stdout=api.raw_io.output_text(name='sdk-path', add_output_log=True),
        step_test_data=lambda: api.raw_io.test_api.stream_output_text(
            '/some/xcode/path')).stdout.strip()
    stdlib = cipd_dir.join('lib', 'libc++.a')
    cxx_include = _get_libcxx_include_path(api)
    env = {
        'CC':
            cipd_dir.join('bin', 'clang'),
        'CXX':
            cipd_dir.join('bin', 'clang++'),
        'AR':
            cipd_dir.join('bin', 'llvm-ar'),
        'CFLAGS':
            '%s %s -nostdinc++ -cxx-isystem %s' %
            (triple, sysroot, cxx_include),
        # TODO(phosek): Use the system libc++ temporarily until we
        # have universal libc++.a for macOS that supports both x86_64
        # and arm64.
        'LDFLAGS':
            '%s %s' % (triple, sysroot),
    }
  else:
    env = {}

  return env


def RunSteps(api, repository):
  src_dir = api.path['start_dir'].join('gn')

  # TODO: Verify that building and linking jemalloc works on OS X and Windows as
  # well.
  with api.step.nest('git'), api.context(infra_steps=True):
    api.step('init', ['git', 'init', src_dir])

    with api.context(cwd=src_dir):
      build_input = api.buildbucket.build_input
      ref = (
          build_input.gitiles_commit.id
          if build_input.gitiles_commit else 'refs/heads/master')
      # Fetch tags so `git describe` works.
      api.step('fetch', ['git', 'fetch', '--tags', repository, ref])
      api.step('checkout', ['git', 'checkout', 'FETCH_HEAD'])
      revision = api.step(
          'rev-parse', ['git', 'rev-parse', 'HEAD'],
          stdout=api.raw_io.output_text()).stdout.strip()
      for change in build_input.gerrit_changes:
        api.step('fetch %s/%s' % (change.change, change.patchset), [
            'git', 'fetch', repository,
            'refs/changes/%s/%s/%s' %
            (str(change.change)[-2:], change.change, change.patchset)
        ])
        api.step('checkout %s/%s' % (change.change, change.patchset),
                 ['git', 'checkout', 'FETCH_HEAD'])

  with api.context(infra_steps=True):
    cipd_dir = api.path['start_dir'].join('cipd')
    pkgs = api.cipd.EnsureFile()
    pkgs.add_package('infra/ninja/${platform}', 'version:1.8.2')
    if api.platform.is_linux or api.platform.is_mac:
      pkgs.add_package('fuchsia/third_party/clang/${platform}', 'integration')
    if api.platform.is_linux:
      pkgs.add_package('fuchsia/third_party/sysroot/linux',
                       'git_revision:c912d089c3d46d8982fdef76a50514cca79b6132',
                       'sysroot')
      # RISCV64 support starts in focal.
      pkgs.add_package('fuchsia/third_party/sysroot/focal',
                       'git_revision:fa7a5a9710540f30ff98ae48b62f2cdf72ed2acd',
                       'sysroot-focal')
    api.cipd.ensure(cipd_dir, pkgs)

  def release_targets():
    if api.platform.is_linux:
      return [
          api.target('linux-amd64'),
          api.target('linux-arm64'),
          api.target('linux-riscv64')
      ]
    elif api.platform.is_mac:
      return [api.target('mac-amd64'), api.target('mac-arm64')]
    else:
      return [api.target.host]

  # The order is important since release build will get uploaded to CIPD.
  configs = [
      {
          'name': 'debug',
          'args': ['-d', '--use-asan', '--use-ubsan'],
          'targets': [api.target.host],
      },
      {
          'name': 'release',
          'args': ['--use-lto', '--use-icf'],
          'targets': release_targets(),
          # TODO: Enable this for OS X and Windows.
          'use_jemalloc': api.platform.is_linux,
      },
  ]

  use_jemalloc = any(c.get('use_jemalloc', False) for c in configs)

  with api.macos_sdk(), api.windows_sdk():
    # Build the jemalloc static library if needed.
    if use_jemalloc:
      # Maps a target.platform string to the location of the corresponding
      # jemalloc static library.
      jemalloc_static_libs = {}

      # Get the list of all target platforms that are listed in `configs`
      # above. Note that this is a list of Target instances, some of them
      # may refer to the same platform string (e.g. linux-amd64).
      #
      # For each platform, a version of jemalloc will be built if necessary,
      # but doing this properly requires having a valid target instance to
      # call _get_compilation_environment. So create a { platform -> Target }
      # map to do that later.
      all_config_platforms = {}
      for c in configs:
        if not c.get('use_jemalloc', False):
          continue
        for t in c['targets']:
          if t.platform not in all_config_platforms:
            all_config_platforms[t.platform] = t

      jemalloc_src_dir = api.path['start_dir'].join('jemalloc')
      with api.step.nest('jemalloc'):
        api.step('init', ['git', 'init', jemalloc_src_dir])
        with api.context(cwd=jemalloc_src_dir, infra_steps=True):
          api.step(
              'fetch',
              ['git', 'fetch', JEMALLOC_GIT_URL, 'refs/tags/' + JEMALLOC_TAG, '--depth=1'])
          api.step('checkout', ['git', 'checkout', 'FETCH_HEAD'])
          api.step('autoconf', ['autoconf'])

        for platform in all_config_platforms:
          # Convert target architecture and os to jemalloc format.
          jemalloc_build_dir = jemalloc_src_dir.join('build-' + platform)

          target = all_config_platforms[platform]
          host_target = api.target.host
          env = _get_compilation_environment(api, target, cipd_dir)

          # Prepare environment for configuring and building jemalloc
          #
          # CFLAGS: -Wno-error is required to avoid warnings that break
          #         compilation with Clang 17.
          #
          # CXXFLAGS: Required to pass the right --target=<target> argument
          #           to the C++ compiler when building jemalloc_cpp.o,
          #           otherwise that file will not be properly cross-compiled.
          #
          # Note that -flto is never passed to CFLAGS/CXXFLAGS/LDFLAGS, since
          # benchmarking shows that this leads to no performance difference
          # for the resulting GN binary.
          #
          env['CFLAGS'] += " -Wno-error"
          env['CXXFLAGS'] = env['CFLAGS']

          api.step('prepare %s build' % platform, ['mkdir', '-p', jemalloc_build_dir])

          with api.step.nest('build jemalloc-' + platform), api.context(
              env=env, cwd=jemalloc_build_dir):
            configure_args = [
              '../configure',
              '--build=' + host_target.triple,
              '--host=' + target.triple,
              '--disable-shared',
              '--enable-static',
              '--disable-syscall',
              '--disable-stats',
            ]
            if platform == 'linux-arm64':
              configure_args.append('--with-lg-page=16')
            api.step('configure', configure_args)
            api.step('build', ['make', '-j%d' % api.platform.cpu_count, 'build_lib_static'])
            jemalloc_static_lib = jemalloc_build_dir.join('lib', 'libjemalloc.a')

          jemalloc_static_libs[platform]  = jemalloc_static_lib

    for config in configs:
      with api.step.nest(config['name']):
        for target in config['targets']:
          env = _get_compilation_environment(api, target, cipd_dir)
          with api.step.nest(target.platform), api.context(
              env=env, cwd=src_dir):
            args = config['args']
            if config.get('use_jemalloc', False):
              args = args[:] + [
                  '--link-lib=%s' % jemalloc_static_libs[target.platform]
              ]

            api.step('generate',
                     ['python3', '-u', src_dir.join('build', 'gen.py')] + args)

            # Windows requires the environment modifications when building too.
            api.step('build',
                     [cipd_dir.join('ninja'), '-C',
                      src_dir.join('out')])

            if target.is_host:
              api.step('test', [src_dir.join('out', 'gn_unittests')])

            if config['name'] != 'release':
              continue

            with api.step.nest('upload'):
              gn = 'gn' + ('.exe' if target.is_win else '')

              if build_input.gerrit_changes:
                # Upload to CAS from CQ.
                api.cas.archive('upload binary to CAS', src_dir.join('out'),
                                src_dir.join('out', gn))
                continue

              cipd_pkg_name = 'gn/gn/%s' % target.platform

              pkg_def = api.cipd.PackageDefinition(
                  package_name=cipd_pkg_name,
                  package_root=src_dir.join('out'),
                  install_mode='copy')
              pkg_def.add_file(src_dir.join('out', gn))
              pkg_def.add_version_file('.versions/%s.cipd_version' % gn)

              cipd_pkg_file = api.path['cleanup'].join('gn.cipd')

              api.cipd.build_from_pkg(
                  pkg_def=pkg_def,
                  output_package=cipd_pkg_file,
              )

              if api.buildbucket.builder_id.project == 'infra-internal':
                cipd_pin = api.cipd.search(cipd_pkg_name,
                                           'git_revision:' + revision)
                if cipd_pin:
                  api.step('Package is up-to-date', cmd=None)
                  continue

                api.cipd.register(
                    package_name=cipd_pkg_name,
                    package_path=cipd_pkg_file,
                    refs=['latest'],
                    tags={
                        'git_repository': repository,
                        'git_revision': revision,
                    },
                )


def GenTests(api):
  for platform in ('linux', 'mac', 'win'):
    yield (api.test('ci_' + platform) + api.platform.name(platform) +
           api.buildbucket.ci_build(
               project='gn',
               git_repo='gn.googlesource.com/gn',
           ))

    yield (api.test('cq_' + platform) + api.platform.name(platform) +
           api.buildbucket.try_build(
               project='gn',
               git_repo='gn.googlesource.com/gn',
           ))

  yield (api.test('cipd_exists') + api.buildbucket.ci_build(
      project='infra-internal',
      git_repo='gn.googlesource.com/gn',
      revision='a' * 40,
  ) + api.step_data(
      'git.rev-parse', api.raw_io.stream_output_text('a' * 40)
  ) + api.step_data(
      'release.linux-amd64.upload.cipd search gn/gn/linux-amd64 git_revision:' +
      'a' * 40,
      api.cipd.example_search('gn/gn/linux-amd64',
                              ['git_revision:' + 'a' * 40])))

  yield (api.test('cipd_register') + api.buildbucket.ci_build(
      project='infra-internal',
      git_repo='gn.googlesource.com/gn',
      revision='a' * 40,
  ) + api.step_data(
      'git.rev-parse', api.raw_io.stream_output_text('a' * 40)
  ) + api.step_data(
      'release.linux-amd64.upload.cipd search gn/gn/linux-amd64 git_revision:' +
      'a' * 40, api.cipd.example_search('gn/gn/linux-amd64', [])))



#!/usr/bin/python3


import argparse
import collections
import json
import os
import subprocess
import sys
import tempfile


class OrderedSet(collections.OrderedDict):

  def add(self, value):
    self[value] = True


def compile_module(module, sources, settings, extras, tmpdir):
  output_file_map = {}
  if settings.whole_module_optimization:
    output_file_map[''] = {
        'object': os.path.join(settings.object_dir, module + '.o'),
        'dependencies': os.path.join(tmpdir, module + '.d'),
    }
  else:
    for source in sources:
      name, _ = os.path.splitext(os.path.basename(source))
      output_file_map[source] = {
          'object': os.path.join(settings.object_dir, name + '.o'),
          'dependencies': os.path.join(tmpdir, name + '.d'),
      }

  for key in ('module_path', 'header_path', 'depfile'):
    path = getattr(settings, key)
    if os.path.exists(path):
      os.unlink(path)
    if key == 'module_path':
      for ext in '.swiftdoc', '.swiftsourceinfo':
        path = os.path.splitext(getattr(settings, key))[0] + ext
        if os.path.exists(path):
          os.unlink(path)
    directory = os.path.dirname(path)
    if not os.path.exists(directory):
      os.makedirs(directory)

  if not os.path.exists(settings.object_dir):
    os.makedirs(settings.object_dir)

  for key in output_file_map:
    path = output_file_map[key]['object']
    if os.path.exists(path):
      os.unlink(path)

  output_file_map_path = os.path.join(tmpdir, module + '.json')
  with open(output_file_map_path, 'w') as output_file_map_file:
    output_file_map_file.write(json.dumps(output_file_map))
    output_file_map_file.flush()

  extra_args = []
  if settings.bridge_header:
    extra_args.extend([
        '-import-objc-header',
        os.path.abspath(settings.bridge_header),
    ])

  if settings.whole_module_optimization:
    extra_args.append('-whole-module-optimization')

  if settings.target:
    extra_args.extend([
        '-target',
        settings.target,
    ])

  if settings.sdk:
    extra_args.extend([
        '-sdk',
        os.path.abspath(settings.sdk),
    ])

  if settings.include_dirs:
    for include_dir in settings.include_dirs:
      extra_args.append('-I' + include_dir)

  process = subprocess.Popen(
      ['swiftc',
       '-parse-as-library',
       '-module-name',
       module,
       '-emit-object',
       '-emit-dependencies',
       '-emit-module',
       '-emit-module-path',
       settings.module_path,
       '-emit-objc-header',
       '-emit-objc-header-path',
       settings.header_path,
       '-output-file-map',
       output_file_map_path,
      ] + extra_args + extras + sources,
      stdout=subprocess.PIPE, stderr=subprocess.PIPE,
      universal_newlines=True)

  stdout, stderr = process.communicate()
  if process.returncode:
    sys.stdout.write(stdout)
    sys.stderr.write(stderr)
    sys.exit(process.returncode)


  depfile_content = collections.OrderedDict()
  for key in output_file_map:
    for line in open(output_file_map[key]['dependencies']):
      output, inputs = line.split(' : ', 2)
      _, ext = os.path.splitext(output)
      if ext == '.o':
        key = output
      else:
        key = os.path.splitext(settings.module_path)[0] + ext
      if key not in depfile_content:
        depfile_content[key] = OrderedSet()
      for path in inputs.split():
        depfile_content[key].add(path)

  with open(settings.depfile, 'w') as depfile:
    for key in depfile_content:
      if not settings.depfile_filter or key in settings.depfile_filter:
        inputs = depfile_content[key]
        depfile.write('%s : %s\n' % (key, ' '.join(inputs)))


def main(args):
  parser = argparse.ArgumentParser(add_help=False)
  parser.add_argument(
      '--module-name',
      help='name of the Swift module')
  parser.add_argument(
      '--include', '-I', action='append', dest='include_dirs',
      help='add directory to header search path')
  parser.add_argument(
      'sources', nargs='+',
      help='Swift source file to compile')
  parser.add_argument(
      '--whole-module-optimization', action='store_true',
      help='enable whole module optimization')
  parser.add_argument(
      '--object-dir', '-o',
      help='path to the generated object files directory')
  parser.add_argument(
      '--module-path', '-m',
      help='path to the generated module file')
  parser.add_argument(
      '--header-path', '-h',
      help='path to the generated header file')
  parser.add_argument(
      '--bridge-header', '-b',
      help='path to the Objective-C bridge header')
  parser.add_argument(
      '--depfile', '-d',
      help='path to the generated depfile')
  parser.add_argument(
      '--depfile-filter', action='append',
      help='limit depfile to those files')
  parser.add_argument(
      '--target', action='store',
      help='generate code for the given target <triple>')
  parser.add_argument(
      '--sdk', action='store',
      help='compile against sdk')

  parsed, extras = parser.parse_known_args(args)
  with tempfile.TemporaryDirectory() as tmpdir:
    compile_module(
        parsed.module_name,
        parsed.sources,
        parsed,
        extras,
        tmpdir)


if __name__ == '__main__':
  sys.exit(main(sys.argv[1:]))



# Copyright 2018 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

from recipe_engine import recipe_api


PLATFORM_TO_TRIPLE = {
  'fuchsia-amd64': 'x86_64-fuchsia',
  'fuchsia-arm64': 'aarch64-fuchsia',
  'linux-amd64': 'x86_64-linux-gnu',
  'linux-arm64': 'aarch64-linux-gnu',
  'linux-riscv64': 'riscv64-linux-gnu',
  'mac-amd64': 'x86_64-apple-darwin',
  'mac-arm64': 'arm64-apple-darwin',
}
PLATFORMS = PLATFORM_TO_TRIPLE.keys()


class Target(object):

  def __init__(self, api, os, arch):
    self.m = api
    self._os = os
    self._arch = arch

  @property
  def is_win(self):
    """Returns True iff the target platform is Windows."""
    return self.os == 'windows'

  @property
  def is_mac(self):
    """Returns True iff the target platform is macOS."""
    return self.os == 'mac'

  @property
  def is_linux(self):
    """Returns True iff the target platform is Linux."""
    return self.os == 'linux'

  @property
  def is_host(self):
    """Returns True iff the target platform is host."""
    return self == self.m.host

  @property
  def os(self):
    """Returns the target os name which will be in:
      * windows
      * mac
      * linux
    """
    return self._os

  @property
  def arch(self):
    """Returns the current CPU architecture."""
    return self._arch

  @property
  def platform(self):
    """Returns the target platform in the <os>-<arch> format."""
    return '%s-%s' % (self.os, self.arch)

  @property
  def triple(self):
    """Returns the target triple."""
    return PLATFORM_TO_TRIPLE[self.platform]

  def __str__(self):
    return self.platform

  def __eq__(self, other):
    if isinstance(other, Target):
      return self._os == other._os and self._arch == other._arch
    return False

  def __ne__(self, other):
    return not self.__eq__(other)


class TargetApi(recipe_api.RecipeApi):

  def __call__(self, platform):
    return Target(self, *platform.split('-', 2))

  @property
  def host(self):
    return Target(self, self.m.platform.name.replace('win', 'windows'), {
        'intel': {
            32: '386',
            64: 'amd64',
        },
        'arm': {
            32: 'armv6',
            64: 'arm64',
        },
    }[self.m.platform.arch][self.m.platform.bits])



# Copyright 2018 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

DEPS = [
    'recipe_engine/cipd',
    'recipe_engine/context',
    'recipe_engine/json',
    'recipe_engine/path',
    'recipe_engine/platform',
    'recipe_engine/step',
]

from recipe_engine.recipe_api import Property
from recipe_engine.config import ConfigGroup, Single

PROPERTIES = {
    '$gn/macos_sdk':
        Property(
            help='Properties specifically for the macos_sdk module.',
            param_name='sdk_properties',
            kind=ConfigGroup(  # pylint: disable=line-too-long
                # XCode build version number. Internally maps to an XCode build id like
                # '9c40b'. See
                #
                #   https://chrome-infra-packages.appspot.com/p/infra_internal/ios/xcode/mac/+/
                #
                # For an up to date list of the latest SDK builds.
                sdk_version=Single(str),

                # The CIPD toolchain tool package and version.
                tool_pkg=Single(str),
                tool_ver=Single(str),
            ),
            default={
                # Check macOS version used in CQ/CI and required version of Xcode.
                # https://xcodereleases.com/
                'sdk_version':
                    '15C500b',  # Xcode 15.2
                'tool_package':
                    'infra/tools/mac_toolchain/${platform}',
                'tool_version':
                    'git_revision:1b9e28e1cfc65541b83def4480835449b60188a1',
            },
        )
}



# Copyright 2018 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

DEPS = [
    'recipe_engine/platform',
]



# Copyright 2019 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
Merges multiple Apple Property List files (.plist) and perform variables
substitutions $(VARIABLE) in the Property List string values.
"""

import argparse
import json
import re
import subprocess
import sys


# Pattern representing a variable to substitue in a string value.
VARIABLE_PATTERN = re.compile(r'\$\(([^)]*)\)')


def GetCommandOutput(command):
  """Returns the output of `command` as a string."""
  return subprocess.check_output(command, encoding='utf-8')


def LoadPlist(plist_path):
  """Loads Apple Property List file at |plist_path|."""
  return json.loads(
      GetCommandOutput(['plutil', '-convert', 'json', '-o', '-', plist_path]))


def SavePlist(plist_path, content, format):
  """Saves |content| as Apple Property List in |format| at |plist_path|."""
  proc = subprocess.Popen(
      ['plutil', '-convert', format, '-o', plist_path, '-'],
      stdin=subprocess.PIPE)
  output, _ = proc.communicate(json.dumps(content).encode('utf-8'))
  if proc.returncode:
    raise subprocess.CalledProcessError(
        proc.returncode,
        ['plutil', '-convert', format, '-o', plist_path, '-'],
        output)


def MergeObjects(obj1, obj2):
  """Merges two objects (either dictionary, list, string or numbers)."""
  if type(obj1) != type(obj2):
    return obj2

  if isinstance(obj2, dict):
    result = dict(obj1)
    for key in obj2:
      value1 = obj1.get(key, None)
      value2 = obj2.get(key, None)
      result[key] = MergeObjects(value1, value2)
    return result

  if isinstance(obj2, list):
    return obj1 + obj2

  return obj2


def MergePlists(plist_paths):
  """Loads and merges all Apple Property List files at |plist_paths|."""
  plist = {}
  for plist_path in plist_paths:
    plist = MergeObjects(plist, LoadPlist(plist_path))
  return plist


def PerformSubstitutions(plist, substitutions):
  """Performs variables substitutions in |plist| given by |substitutions|."""
  if isinstance(plist, dict):
    result = dict(plist)
    for key in plist:
      result[key] = PerformSubstitutions(plist[key], substitutions)
    return result

  if isinstance(plist, list):
    return [ PerformSubstitutions(item, substitutions) for item in plist ]

  if isinstance(plist, str):
    result = plist
    while True:
      match = VARIABLE_PATTERN.search(result)
      if not match:
        break

      extent = match.span()
      expand = substitutions[match.group(1)]
      result = result[:extent[0]] + expand + result[extent[1]:]
    return result

  return plist


def PerformSubstitutionsFrom(plist, substitutions_path):
  """Performs variable substitutions in |plist| from |substitutions_path|."""
  with open(substitutions_path) as substitutions_file:
    return PerformSubstitutions(plist, json.load(substitutions_file))


def ParseArgs(argv):
  """Parses command line arguments."""
  parser = argparse.ArgumentParser(
      description=__doc__,
      formatter_class=argparse.ArgumentDefaultsHelpFormatter)

  parser.add_argument(
      '-s', '--substitutions',
      help='path to a JSON file containing variable substitutions')
  parser.add_argument(
      '-f', '--format', default='json', choices=('json', 'binary1', 'xml1'),
      help='format of the generated file')
  parser.add_argument(
      '-o', '--output', default='-',
      help='path to the result; - means stdout')
  parser.add_argument(
      'inputs', nargs='+',
      help='path of the input files to merge')

  return parser.parse_args(argv)


def main(argv):
  args = ParseArgs(argv)

  data = MergePlists(args.inputs)
  if args.substitutions:
    data = PerformSubstitutionsFrom(
        data, args.substitutions)

  SavePlist(args.output, data, args.format)


if __name__ == '__main__':
  sys.exit(main(sys.argv[1:]))



# Copyright 2018 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

from contextlib import contextmanager

from recipe_engine import recipe_api


class WindowsSDKApi(recipe_api.RecipeApi):
  """API for using Windows SDK distributed via CIPD."""

  def __init__(self, sdk_properties, *args, **kwargs):
    super(WindowsSDKApi, self).__init__(*args, **kwargs)

    self._sdk_package = sdk_properties['sdk_package']
    self._sdk_version = sdk_properties['sdk_version']

  @contextmanager
  def __call__(self):
    """Setups the Windows SDK environment.

    This call is a no-op on non-Windows platforms.

    Raises:
        StepFailure or InfraFailure.
    """
    if not self.m.platform.is_win:
      yield
      return

    with self.m.context(infra_steps=True):
      sdk_dir = self._ensure_sdk()

    with self.m.context(**self._sdk_env(sdk_dir)):
      try:
        yield
      finally:
        # cl.exe automatically starts background mspdbsrv.exe daemon which
        # needs to be manually stopped so Swarming can tidy up after itself.
        self.m.step('taskkill mspdbsrv',
                    ['taskkill.exe', '/f', '/t', '/im', 'mspdbsrv.exe'])

  def _ensure_sdk(self):
    """Ensures the Windows SDK CIPD package is installed.

    Returns the directory where the SDK package has been installed.

    Args:
      path (path): Path to a directory.
      version (str): CIPD instance ID, tag or ref.
    """
    sdk_dir = self.m.path['cache'].join('windows_sdk')
    pkgs = self.m.cipd.EnsureFile()
    pkgs.add_package(self._sdk_package, self._sdk_version)
    self.m.cipd.ensure(sdk_dir, pkgs)
    return sdk_dir

  def _sdk_env(self, sdk_dir):
    """Constructs the environment for the SDK.

    Returns environment and environment prefixes.

    Args:
      sdk_dir (path): Path to a directory containing the SDK.
    """
    env = {}
    env_prefixes = {}

    # Load .../Windows Kits/10/bin/SetEnv.${arch}.json to extract the required
    # environment. It contains a dict that looks like this:
    # {
    #   "env": {
    #     "VAR": [["x"], ["y"]],
    #     ...
    #   }
    # }
    # All these environment variables need to be added to the environment
    # for the compiler and linker to work.
    filename = 'SetEnv.%s.json' % {32: 'x86', 64: 'x64'}[self.m.platform.bits]
    step_result = self.m.json.read(
        'read %s' % filename,
        sdk_dir.join('Windows Kits', '10', 'bin', filename),
        step_test_data=lambda: self.m.json.test_api.output({
            'env': {
                'PATH': [['Windows Kits', '10', 'bin', '10.0.19041.0', 'x64']],
                'VSINSTALLDIR': [['.\\']],
            },
        }))
    data = step_result.json.output.get('env')
    for key in data:
      results = ['%s' % sdk_dir.join(*e) for e in data[key]]

      # PATH is special-cased because we don't want to overwrite other things
      # like C:\Windows\System32. Others are replacements because prepending
      # doesn't necessarily makes sense, like VSINSTALLDIR.
      if key.lower() == 'path':
        env_prefixes[key] = results
      else:
        env[key] = ';'.join(results)

    return {'env': env, 'env_prefixes': env_prefixes}



# Copyright 2019 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""Collects information about the SDK and return them as JSON file."""

import argparse
import json
import os
import re
import subprocess
import sys

# Patterns used to extract the Xcode version and build version.
XCODE_VERSION_PATTERN = re.compile(r'Xcode (\d+)\.(\d+)')
XCODE_BUILD_PATTERN = re.compile(r'Build version (.*)')


def GetCommandOutput(command):
  """Returns the output of `command` as a string."""
  return subprocess.check_output(command, encoding='utf-8')


def GetAppleCpuName(target_cpu):
  """Returns the name of the |target_cpu| using Apple's convention."""
  return {
      'x64': 'x86_64',
      'arm': 'armv7',
      'x86': 'i386'
  }.get(target_cpu, target_cpu)


def GetPlatform(target_environment):
  """Returns the platform for |target_environment|."""
  return {
      'simulator': 'iphonesimulator',
      'device': 'iphoneos'
  }[target_environment]


def GetPlaformDisplayName(target_environment):
  """Returns the platform display name for |target_environment|."""
  return {
      'simulator': 'iPhoneSimulator',
      'device': 'iPhoneOS'
  }[target_environment]


def ExtractOSVersion():
  """Extract the version of macOS of the current machine."""
  return GetCommandOutput(['sw_vers', '-buildVersion']).strip()


def ExtractXcodeInfo():
  """Extract Xcode version and build version."""
  version, build = None, None
  for line in GetCommandOutput(['xcodebuild', '-version']).splitlines():
    match = XCODE_VERSION_PATTERN.search(line)
    if match:
      major, minor = match.group(1), match.group(2)
      version = major.rjust(2, '0') + minor.ljust(2, '0')
      continue

    match = XCODE_BUILD_PATTERN.search(line)
    if match:
      build = match.group(1)
      continue

  assert version is not None and build is not None
  return version, build


def ExtractSDKInfo(info, sdk):
  """Extract information about the SDK."""
  return GetCommandOutput(['xcrun', '--sdk', sdk, '--show-sdk-' + info]).strip()


def GetDeveloperDir():
  """Returns the developer dir."""
  return GetCommandOutput(['xcode-select', '-print-path']).strip()


def GetSDKInfoForCpu(target_cpu, environment, sdk_version, deployment_target):
  """Returns a dictionary with information about the SDK."""
  platform = GetPlatform(environment)
  sdk_version = sdk_version or ExtractSDKInfo('version', platform)
  deployment_target = deployment_target or sdk_version

  target = target_cpu + '-apple-ios' + deployment_target
  if environment == 'simulator':
    target = target + '-simulator'

  xcode_version, xcode_build = ExtractXcodeInfo()
  effective_sdk = platform + sdk_version

  sdk_info = {}
  sdk_info['compiler'] = 'com.apple.compilers.llvm.clang.1_0'
  sdk_info['is_simulator'] = environment == 'simulator'
  sdk_info['macos_build'] = ExtractOSVersion()
  sdk_info['platform'] = platform
  sdk_info['platform_name'] = GetPlaformDisplayName(environment)
  sdk_info['sdk'] = effective_sdk
  sdk_info['sdk_build'] = ExtractSDKInfo('build-version', effective_sdk)
  sdk_info['sdk_path'] = ExtractSDKInfo('path', effective_sdk)
  sdk_info['toolchain_path'] = os.path.join(
      GetDeveloperDir(), 'Toolchains/XcodeDefault.xctoolchain')
  sdk_info['sdk_version'] = sdk_version
  sdk_info['target'] = target
  sdk_info['xcode_build'] = xcode_build
  sdk_info['xcode_version'] = xcode_version

  return sdk_info


def ParseArgs(argv):
  """Parses command line arguments."""
  parser = argparse.ArgumentParser(
      description=__doc__,
      formatter_class=argparse.ArgumentDefaultsHelpFormatter)

  parser.add_argument(
      '-t', '--target-cpu', default='x64',
      choices=('x86', 'x64', 'arm', 'arm64'),
      help='target cpu')
  parser.add_argument(
      '-e',
      '--target-environment',
      default='simulator',
      choices=('simulator', 'device'),
      help='target environment')
  parser.add_argument(
      '-s', '--sdk-version',
      help='version of the sdk')
  parser.add_argument(
      '-d', '--deployment-target',
      help='iOS deployment target')
  parser.add_argument(
      '-o', '--output', default='-',
      help='path of the output file to create; - means stdout')

  return parser.parse_args(argv)


def main(argv):
  args = ParseArgs(argv)

  sdk_info = GetSDKInfoForCpu(
      GetAppleCpuName(args.target_cpu), args.target_environment,
      args.sdk_version, args.deployment_target)

  if args.output == '-':
    sys.stdout.write(json.dumps(sdk_info))
  else:
    with open(args.output, 'w') as output:
      output.write(json.dumps(sdk_info))


if __name__ == '__main__':
  sys.exit(main(sys.argv[1:]))
